@@ -40,53 +40,49 @@ async def treat_response(rsp: ClientResponse) -> Any:
 	else:
 		return await rsp.text()
 
-async def request(self: "NovelAI_API", request_method: MethodDescriptorType, endpoint: str, data: Optional[Union[Dict[str, Any], str]], need_login: bool) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
-	"""
-	:param url: Endpoint of the request
-	:param request_method: Method of the reqest from ClientSession
-	:param data: Data to pass to the request_method if needed
-	:param need_login: Tell if the request needs the authorization token
-	"""
-
-	url = f"{self._BASE_ADDRESS}{endpoint}"
-
-	headers = self.base_headers.copy()
-	if need_login:
-		assert self._token is not None, f"You need to be log in to use the endpoint {endpoint}"
-		headers["Authorization"] = f"Bearer {self._token}"
-
-	try:
-		if type(data) is dict:	# data transforms dict in str
-			async with request_method(url, json = data, headers = headers) as rsp:
-				return (rsp, await treat_response(rsp))
-		else:
-			async with request_method(url, data = data, headers = headers) as rsp:
-				return (rsp, await treat_response(rsp))
-	except (ClientError, HttpProcessingError) as e:
-		# FIXME: when are these errors triggered ? raise_as_error ?
-		return NovelAIError(0, "Unknown error")
-
-async def get(self: "NovelAI_API", endpoint: str, need_login: bool = True) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
-	return await request(self, self._session.get, endpoint, None, need_login)
-
-async def post(self: "NovelAI_API", endpoint: str, data: Dict[str, Any], need_login: bool = True) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
-	return await request(self, self._session.post, endpoint, data, need_login)
-
-async def put(self: "NovelAI_API", endpoint: str, data: Union[Dict[str, Any], str],need_login: bool = True) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
-	return await request(self, self._session.put, endpoint, data, need_login)
-
-async def patch(self: "NovelAI_API", endpoint: str, data: Dict[str, Any], need_login: bool = True) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
-	return await request(self, self._session.patch, endpoint, data, need_login)
-
-async def delete(self: "NovelAI_API", endpoint: str, need_login: bool = True) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
-	return await request(self, self._session.delete, endpoint, None, need_login)
-
 #=== API ===#
 class Low_Level:
 	_parent: "NovelAI_API"
+	_session: ClientSession
 
 	def __init__(self, parent: "NovelAI_API"):
 		self._parent = parent
+		self._session = parent._session
+
+	async def request(self, request_method: MethodDescriptorType, endpoint: str, data: Optional[Union[Dict[str, Any], str]]) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
+		"""
+		:param url: Endpoint of the request
+		:param request_method: Method of the reqest from ClientSession
+		:param data: Data to pass to the request_method if needed
+		"""
+
+		url = f"{self._parent._BASE_ADDRESS}{endpoint}"
+
+		try:
+			if type(data) is dict:	# data transforms dict in str
+				async with request_method(url, json = data) as rsp:
+					return (rsp, await treat_response(rsp))
+			else:
+				async with request_method(url, data = data) as rsp:
+					return (rsp, await treat_response(rsp))
+		except (ClientError, HttpProcessingError) as e:
+			# FIXME: when are these errors triggered ? raise_as_error ?
+			return NovelAIError(0, "Unknown error")
+
+	async def get(self, endpoint: str) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
+		return await self.request(self._session.get, endpoint, None)
+
+	async def post(self, endpoint: str, data: Dict[str, Any]) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
+		return await self.request(self._session.post, endpoint, data)
+
+	async def put(self, endpoint: str, data: Union[Dict[str, Any], str],need_login: bool = True) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
+		return await self.request(self._session.put, endpoint, data)
+
+	async def patch(self, endpoint: str, data: Dict[str, Any]) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
+		return await self.request(self._session.patch, endpoint, data)
+
+	async def delete(self, endpoint: str) -> Union[Tuple[ClientResponse, Any], NovelAIError]:
+		return await self.request(self._session.delete, endpoint, None)
 
 	async def is_reachable(self) -> Union[bool, NovelAIError]:
 		"""
@@ -94,7 +90,7 @@ class Low_Level:
 
 		:return: True if reachable, False if not, NovelAIError on error
 		"""
-		rsp, content = await get(self._parent, "/", False)
+		rsp, content = await self.get("/")
 		return treat_response_bool(rsp, content, 200)
 
 	async def register(self, recapcha: str, access_key: str, email: Optional[str], giftkey: Optional[str] = None) -> Union[bool, NovelAIError]:
@@ -124,7 +120,7 @@ class Low_Level:
 		if giftkey is not None:
 			data["giftkey"] = giftkey
 
-		rsp, content = await post(self._parent, "/user/register", data, False)
+		rsp, content = await self.post("/user/register", data)
 		rsp = treat_response_bool(rsp, content, 201)
 		if type(rsp) is not NovelAIError:
 			# FIXME: handle cases where the response is corrupted
@@ -145,11 +141,8 @@ class Low_Level:
 
 		assert len(access_key) == 64, f"access_key should be 64 characters, got length of {len(access_key)}"
 
-		rsp, content = await post(self._parent, "/user/login", { "key": access_key }, False)
+		rsp, content = await self.post("/user/login", { "key": access_key })
 		rsp = treat_response_object(rsp, content, 201)
-		if type(rsp) is not NovelAIError:
-			# FIXME: handle cases where the response is corrupted
-			self._parent._token = rsp["accessToken"]
 
 		return rsp
 
@@ -160,13 +153,13 @@ class Low_Level:
 		assert len(current_key) == 64, f"Current access key should be 64 characters, got length of {len(current_key)}"
 		assert len(new_key) == 64, f"New access key should be 64 characters, got length of {len(new_key)}"
 
-		rsp, content = await post(self._parent, "/user/change-access-key", { "currentAccessKey": current_key, "newAccessKey": new_key })
+		rsp, content = await self.post("/user/change-access-key", { "currentAccessKey": current_key, "newAccessKey": new_key })
 		return treat_response_bool(rsp, content, 200)
 
 	async def request_account_recovery(self, email: str) -> Union[bool, NovelAIError]:
 		assert type(email) is str, f"Expected type 'str' for email, but got type '{type(email)}'"
 
-		rsp, content = await post(self._parent, "/user/recovery/request", { "email": email }, False)
+		rsp, content = await self.post("/user/recovery/request", { "email": email })
 		return treat_response_bool(rsp, content, 202)
 
 	async def recover_account(self, recovery_token: str, new_key: str, delete_content: bool = False) -> Union[bool, NovelAIError]:
@@ -177,35 +170,35 @@ class Low_Level:
 		assert 16 <= len(recovery_token), f"Recovery token should be at least 16 characters, got length of {len(recovery_token)}"
 		assert len(new_key) == 64, f"New access key should be 64 characters, got length of {len(new_key)}"
 
-		rsp, content = await post(self._parent, "//user/recovery/recover", { "recoveryToken": recovery_token, "newAccessKey": new_key, "deleteContent": delete_content }, False)
+		rsp, content = await self.post("//user/recovery/recover", { "recoveryToken": recovery_token, "newAccessKey": new_key, "deleteContent": delete_content })
 		return treat_response_bool(rsp, content, 201)
 
 	async def delete_account(self) -> Union[bool, NovelAIError]:
-		rsp, content = await post(self._parent, "/user/delete", None)
+		rsp, content = await self.post("/user/delete", None)
 		return treat_response_bool(rsp, content, 200)
 
 	async def get_priority(self) -> Union[Dict[str, Any], NovelAIError]:
-		rsp, content = await get(self._parent, "/user/priority")
+		rsp, content = await self.get("/user/priority")
 		return treat_response_object(rsp, content, 200)
 
 	async def get_subscription(self) -> Union[Dict[str, Any], NovelAIError]:
-		rsp, content = await get(self._parent, "/user/subscription")
+		rsp, content = await self.get("/user/subscription")
 		return treat_response_object(rsp, content, 200)
 
 	async def get_keystore(self) -> Union[Dict[str, str], NovelAIError]:
-		rsp, content = await get(self._parent, "/user/keystore")
+		rsp, content = await self.get("/user/keystore")
 		return treat_response_object(rsp, content, 200)
 
 	async def set_keystore(self, value: str) -> Union[bool, NovelAIError]:
 		assert type(value) is str, f"Expected type 'str' for keystore, but got type '{type(value)}'"
 
-		rsp, content = await put(self._parent, "/user/keystore", { "keystore": value })
+		rsp, content = await self.put("/user/keystore", { "keystore": value })
 		return treat_response_object(rsp, content, 200)
 
 	async def download_objects(self, object_type: str) -> Union[Dict[str, List[Dict[str, Union[str, int]]]], NovelAIError]:
 		assert type(object_type) is str, f"Expected type 'str' for object_type, but got type '{type(object_type)}'"
 
-		rsp, content = await get(self._parent, f"/user/objects/{object_type}")
+		rsp, content = await self.get(f"/user/objects/{object_type}")
 		return treat_response_object(rsp, content, 200)
 
 	async def upload_objects(self, object_type: str, meta: str, data: str) -> Union[bool, NovelAIError]:
@@ -215,14 +208,14 @@ class Low_Level:
 
 		assert len(meta) <= 128, f"Meta should be at most 128 characters, got length of {len(meta)}"
 
-		rsp, content = await put(self._parent, f"/user/objects/{object_type}", { "meta": meta, "data": data })
+		rsp, content = await self.put(f"/user/objects/{object_type}", { "meta": meta, "data": data })
 		return treat_response_bool(rsp, content, 200)
 
 	async def download_object(self, object_type: str, object_id: str) -> Union[Dict[str, Union[str, int]], NovelAIError]:
 		assert type(object_type) is str, f"Expected type 'str' for object_type, but got type '{type(object_type)}'"
 		assert type(object_id) is str, f"Expected type 'str' for object_id, but got type '{type(object_id)}'"
 
-		rsp, content = await get(self._parent, f"/user/objects/{object_type}/{object_id}")
+		rsp, content = await self.get(f"/user/objects/{object_type}/{object_id}")
 		return treat_response_object(rsp, content, 200)
 
 	async def upload_object(self, object_type: str, object_id: str, meta: str, data: str) -> Union[bool, NovelAIError]:
@@ -233,37 +226,37 @@ class Low_Level:
 
 		assert len(meta) <= 128, f"Meta should be at most 128 characters, got length of {len(meta)}"
 
-		rsp, content = await patch(self._parent, f"/user/objects/{object_type}/{object_id}", { "meta": meta, "data": data })
+		rsp, content = await self.patch(f"/user/objects/{object_type}/{object_id}", { "meta": meta, "data": data })
 		return treat_response_bool(rsp, content, 200)
 
 	async def delete_object(self, object_type: str, object_id: str) -> Union[Dict[str, Union[str, int]], NovelAIError]:
 		assert type(object_type) is str, f"Expected type 'str' for object_type, but got type '{type(object_type)}'"
 		assert type(object_id) is str, f"Expected type 'str' for object_id, but got type '{type(object_id)}'"
 
-		rsp, content = await delete(self._parent, f"/user/objects/{object_type}/{object_id}")
+		rsp, content = await self.delete(f"/user/objects/{object_type}/{object_id}")
 		return treat_response_object(rsp, content, 200)
 
 	async def get_settings(self) -> Union[str, NovelAIError]:
-		rsp, content = await get(self._parent, "/user/clientsettings")
+		rsp, content = await self.get("/user/clientsettings")
 		return treat_response_object(rsp, content, 200)
 
 	async def set_settings(self, value: str) -> Union[bool, NovelAIError]:
 		assert type(value) is str, f"Expected type 'str' for value, but got type '{type(value)}'"
 
-		rsp, content = await put(self._parent, "/user​/clientsettings", value)
+		rsp, content = await self.put("/user​/clientsettings", value)
 		return treat_response_bool(rsp, content, 200)
 
 	async def bind_subscription(self, payment_processor: str, subscription_id: str) -> Union[bool, NovelAIError]:
 		assert type(payment_processor) is str, f"Expected type 'str' for payment_processor, but got type '{type(payment_processor)}'"
 		assert type(subscription_id) is str, f"Expected type 'str' for subscription_id, but got type '{type(subscription_id)}'"
 
-		rsp, content = await post(self._parent, "/user/subscription/bind", { "paymentProcessor": payment_processor, "subscriptionId": subscription_id })
+		rsp, content = await self.post("/user/subscription/bind", { "paymentProcessor": payment_processor, "subscriptionId": subscription_id })
 		return treat_response_bool(rsp, content, 201)
 
 	async def change_subscription(self, new_plan: str) -> Union[bool, NovelAIError]:
 		assert type(new_plan) is str, f"Expected type 'str' for new_plan, but got type '{type(new_plan)}'"
 
-		rsp, content = await post(self._parent, "​/user​/subscription​/change", { "newSubscriptionPlan": new_plan })
+		rsp, content = await self.post("​/user​/subscription​/change", { "newSubscriptionPlan": new_plan })
 		return treat_response_bool(rsp, content, 200)
 
 	async def generate(self, input: str, model: str, params: Dict[str, Any]) -> Union[Dict[str, str], NovelAIError]:
@@ -284,7 +277,7 @@ class Low_Level:
 
 		args = { "input": input, "model": model, "parameters": params }
 
-		rsp, content = await post(self._parent, "/ai/generate", args)
+		rsp, content = await self.post("/ai/generate", args)
 		return treat_response_object(rsp, content, 201)
 
 	async def generate_stream(self):
@@ -310,7 +303,7 @@ class Low_Level:
 		assert type(name) is str, f"Expected type 'str' for name, but got type '{type(name)}'"
 		assert type(desc) is str, f"Expected type 'str' for desc, but got type '{type(desc)}'"
 
-		rsp, content = await post(self._parent, "​/ai​/module​/train", { "data": data, "lr": rate, "steps": steps, "name": name, "description": desc })
+		rsp, content = await self.post("​/ai​/module​/train", { "data": data, "lr": rate, "steps": steps, "name": name, "description": desc })
 		return treat_response_object(rsp, content, 201)
 
 	async def get_modules(self) -> Union[List[Dict[str, Any]], NovelAIError]:
@@ -318,7 +311,7 @@ class Low_Level:
 		:return: List of modules saved on the logged account
 		"""
 
-		rsp, content = await get(self._parent, "/ai/module/all")
+		rsp, content = await self.get("/ai/module/all")
 		return treat_response_object(rsp, content, 200)
 
 	async def get_module(self, module_id: str) -> Union[Dict[str, Any], NovelAIError]:
@@ -330,7 +323,7 @@ class Low_Level:
 
 		assert type(module_id) is str, f"Expected type 'str' for module_id, but got type '{type(module_id)}'"
 
-		rsp, content = await get(self._parent, f"/ai/module/{module_id}")
+		rsp, content = await self.get(f"/ai/module/{module_id}")
 		return treat_response_object(rsp, content, 200)
 
 	async def delete_module(self, module_id: str) -> Union[Dict[str, Any], NovelAIError]:
@@ -344,5 +337,5 @@ class Low_Level:
 
 		assert type(module_id) is str, f"Expected type 'str' for module_id, but got type '{type(module_id)}'"
 
-		rsp, content = await delete(self._parent, f"/ai/module/{module_id}")
+		rsp, content = await self.delete(f"/ai/module/{module_id}")
 		return treat_response_object(rsp, content, 200)
\ No newline at end of file
